import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.*;
import java.awt.image.*;

class ImageButton extends Component {
  ActionListener actionListener;     // Post action events to listeners
  protected boolean pressed = false; // true if the button is detented.
  String label;                      // The Button's text
  Image imgOn, imgOff;

  //public ImageButton() {
  //  this(null, null, "?");
  //}

  //public ImageButton(String label) {
  //  this(, , label);
  //}

 /**
   * Constructs an ImageButton with the specified label and images
   * @param label the label of the button
   */
  public ImageButton(Image imOff, Image imOn, String label) {
    this.label = label;
    this.imgOn = imOn;
    this.imgOff= imOff;
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }

 /**
   * gets the label
   * @see setLabel
   */
  public String getLabel() {
      return label;
  }

  /**
   * sets the label
   * @see getLabel
   */
  public void setLabel(String label) {
      this.label = label;
      invalidate();
      repaint();
  }
		
  /**
   * sets the image of pressed button
   * @see getImageOn 
   * @see getImageOff 
   * @see setImageOff 
   */
  public void setImageOn(Image imgOn) {
      this.imgOn = imgOn;
      invalidate();
      repaint();
  }

 /**
   * gets the image of pressed button
   * @see setImageOn
   * @see setImageOff
   * @see getImageOff 
   */
  public Image getImageOn() {
      return imgOn;
  }
  
  /**
   * sets the image of not pressed button
   * @see getImageOn 
   * @see getImageOff 
   * @see setImageOn 
   */
  public void setImageOff(Image imgOff) {
      this.imgOff = imgOff;
      invalidate();
      repaint();
  }

 /**
   * gets the image of not pressed button
   * @see setImageOn
   * @see setImageOff
   * @see getImageOn 
   */
  public Image getImageOff() {
      return imgOff;
  }

 /**
   * paints the ImageButton
   */
 public void paint(Graphics g) {
      // draw the label centered in the button
      Font f = getFont();
      if(f != null) {
	  FontMetrics fm = getFontMetrics(getFont());
	  g.setColor(getForeground());
	  g.drawString(label, 0, 15);
		       //s/2 - fm.stringWidth(label)/2,
		       //s/2 + fm.getMaxDescent());
      }
      // paint the interior of the button
      if(pressed) {
	//if(imgOn!=null)
	  g.drawImage(imgOn, 0, 0, this);
      } else {
	  g.drawImage(imgOff, 0, 0, this);
      }
  }
	
  /**
   * The preferred size of the button. 
   */
  public Dimension getPreferredSize() {
    if(imgOff != null) {
		  return new Dimension(165, 37);
		} else {
		  return new Dimension(165, 37);
		}
  }
  
  /**
   * The minimum size of the button. 
   */
  public Dimension getMinimumSize() {
      return getPreferredSize();
  }

  /**
   * Adds the specified action listener to receive action events
   * from this button.
   * @param listener the action listener
   */
   public void addActionListener(ActionListener listener) {
       actionListener = AWTEventMulticaster.add(actionListener, listener);
       enableEvents(AWTEvent.MOUSE_EVENT_MASK);
   }
 
   /**
    * Removes the specified action listener so it no longer receives
    * action events from this button.
    * @param listener the action listener
    */
   public void removeActionListener(ActionListener listener) {
       actionListener = AWTEventMulticaster.remove(actionListener, listener);
   }

  /**
   * Determine if click was inside round button.
   */
   public boolean contains(int x, int y) {
    
    //return true; // I always have rectangular buttonc, no stars, no circles
    return ((x<getSize().width)&&(y<getSize().height)&&(x>=0)&&(y>=0));
   }
   
   /**
    * Paints the button and distribute an action event to all listeners.
    */
   public void processMouseEvent(MouseEvent e) {
       Graphics g;
       switch(e.getID()) {
          case MouseEvent.MOUSE_PRESSED:
	    // render myself inverted....
	    pressed = true;

            // Repaint might flicker a bit. To avoid this, you can use
            // double buffering (see the Gauge example).
	    repaint(); 
	    break;
          case MouseEvent.MOUSE_RELEASED:
	    if(actionListener != null) {
	       actionListener.actionPerformed(new ActionEvent(
		   this, ActionEvent.ACTION_PERFORMED, label));
	    }
	    // render myself normal again
	    if(pressed == true) {
		pressed = false;

                // Repaint might flicker a bit. To avoid this, you can use
                // double buffering (see the Gauge example).
		repaint();
	    }
	    break;
          case MouseEvent.MOUSE_ENTERED:

	    break;
          case MouseEvent.MOUSE_EXITED:
	    if(pressed == true) {
		// Cancel! Don't send action event.
		pressed = false;

                // Repaint might flicker a bit. To avoid this, you can use
                // double buffering (see the Gauge example).
		repaint();

		// Note: for a more complete button implementation,
		// you wouldn't want to cancel at this point, but
		// rather detect when the mouse re-entered, and
		// re-highlight the button. There are a few state
		// issues that that you need to handle, which we leave
		// this an an excercise for the reader (I always
		// wanted to say that!)
	    }
	    break;
       }
       super.processMouseEvent(e);
   }
	
	
	
}
