import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;
import java.lang.String;


/**
* Trida slouzici pro vytvoreni platna. Zachycuje udalosti "pohyb mysi"
* a "stisk tlacitka" - proto implementuje uvedna rozhrani.
* @see java.awt.event.MouseMotionListener
* @see java.awt.event.ActionListener
*/
class Platno extends Canvas
implements MouseMotionListener, ActionListener, MouseListener {
  Graphics offGraphics; 		// skryty graficky kontext
  Image offImage;			// skryty obrazek (buffer)
  int oldx, oldy;			// "stare" souradnice
  static int velBodu = 15;   	// neboli ctverecku 15
  static int sirka = 9;   	// neboli pocet ctverecku v radku
  static int vyska = 15;   	// neboli pocet ctverecku v sloupci
  static int sirkaPlatna = sirka * velBodu; // v ctvereckach
  static int vyskaPlatna = vyska * velBodu; // v ctvereckach
  Digits sd;

  /** Konstruktor */
  public Platno(Digits s) {
    sd = s;
    setBackground(Color.white);	// nastaveni pozadi
    addMouseMotionListener(this);	// regisrace sebe (this), jako prijemce udalosti "pohyb mysi"
    addMouseListener(this);	// regisrace sebe (this), jako prijemce udalosti "kliknuti mysi"
  }
  
  public Color getBarvu()            { return offGraphics.getColor(); }
  public void  setBarvu(Color barva) { offGraphics.setColor(barva); }
  
  //pozustatky z puvodniho zdrojaku od P.Tomana - nefunkcni
  public Dimension getMinimumSize() {return new Dimension(sirkaPlatna, vyskaPlatna);}
  public Dimension getPreferredSize() {return getMinimumSize();	}
  
  // nutno naimplementovat vsechny metody rozhrani
  // Je to z interface -> nutno definovat
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
  public void mouseMoved(MouseEvent e) {}
  	
  /**
  * Metoda implemtuje obsluhu udalosti od mysi. Je volana
  * z metod mouseClicked() a mouseDragged()
  */
  public void draw(MouseEvent e) {
    if (offGraphics != null) { 	// je-li mozno provadet vystup, vykresli caru
      int x, y;
      x = (velBodu * ((int) e.getX()/(velBodu)));
      y = (velBodu * ((int) e.getY()/(velBodu)));
      if (!sd.checkMustr(x, y))
	offGraphics.fillRect(x+1, y+1, velBodu-1, velBodu-1);
      else {
        Color tmp = offGraphics.getColor();
        offGraphics.setColor(Color.gray);
	offGraphics.fillRect(x+1, y+1, velBodu-1, velBodu-1);
        offGraphics.setColor(tmp);
      }
    }
    repaint();		// prekreslit platno !
  }
  
  /**
  * Metoda mouseClicked() je volana pri stisku mysi. Rozenava se tu 
  * ktere tlacitkoa mysi bylo stisknuto a podle toho se nastavi
  * barva na kresleni nabo na mazani.
  */
  public void mousePressed(MouseEvent e) {
    if ((e.getModifiers() & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK)
      offGraphics.setColor(Color.black);
    else
      if ( ((e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK)||
  	 ((e.getModifiers() & InputEvent.BUTTON2_MASK) == InputEvent.BUTTON2_MASK)  ) {
   	      offGraphics.setColor(Color.white);
      }
  }
  
  /**
  * Metoda mouseClicked() je volana pri kliknuti mysi
  */
  public void mouseClicked(MouseEvent e) {
    draw(e);
  }
  
  /**
  * Metoda mouseDragged() je volana pri pohybu a soucasnem drzeni
  * tlacitka mysi.
  */
  public void mouseDragged(MouseEvent e) {
    draw(e);
  }
  
  /**
  * Metoda actionPerformed() je volana pri stisku tlacitka "Clear".
  */
  public void actionPerformed(ActionEvent e) {
    offImage = null; 	// zruseni skryteho buffer (bude provedeno vymazani)
    repaint();		// prekresli platno!
  }
  
  /** 
  * Metoda update() je volana pri prekreslovani komponenty.
  * Standartne vymaze pozadi a vola metodu paint(). 
  * Je predefinovana, aby nedochazelo k blikani.
  */
  public void update(Graphics g) {
    //sd.play(sd.getCodeBase(), "ok.au"); // :-)
    paint(g);
  }
  
  /**
  * Metoda paint() provadi vykresleni obsahu platna - prekopiruje
  * skryty buffer do grafickeho kontextu komponenty (paramter g);
  */
  public void paint(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    if (offImage == null) {
      offImage = createImage(width, height);	// vytvoreni prazdneho bufferu
      offGraphics = offImage.getGraphics();
      int i;
      offGraphics.setColor(Color.gray);
      for (i=0; i<vyska; i++) 
        offGraphics.drawLine(0, i * velBodu, sirkaPlatna, i * velBodu);
      for (i=0; i<sirka; i++) 
        offGraphics.drawLine(i * velBodu, 0, i * velBodu, vyskaPlatna);
      sd.zobrazVzor(Color.lightGray, 8);
    }
    g.drawImage(offImage, 0, 0, this);		// zobrazeni bufferu do grafickeho kontextu komponenty
  }
  
  /***************************************************************************/
  /***************************************************************************/
  /***************************************************************************/

      
  /**
  * Metoda kresli jeden ctverecek na platno. Ctvereckem se mysli
  * jeden ukazkovy "bod" (velikost 9x9 pixelu)
  */
  public void kresliCtverec(int i, int j, Color c, boolean vyplneny, boolean vykreslovatPoJednom) {
    Color tmp = offGraphics.getColor();
    offGraphics.setColor(c);
    if (vyplneny)
      offGraphics.fillRect(velBodu*(i)+1, velBodu*(j)+1, velBodu-1, velBodu-1);
    else  
      offGraphics.drawRect(velBodu*(i)+1, velBodu*(j)+1, velBodu-2, velBodu-2);
    offGraphics.setColor(tmp);
    if (vykreslovatPoJednom)
      update(getGraphics());          //repaint(); // repaint nefacha pri SLEEPu
  }
/*  
  public void b() {
    Color tmp = offGraphics.getColor();
    offGraphics.setColor(Color.green);
    offGraphics.setFont(new Font("Helvetica", Font.ITALIC, 24));
    offGraphics.drawString("foo", 100, 100);
    offGraphics.setColor(tmp);
    update(getGraphics());    
  }
*/
  /**
  * Metoda ulozi do pole nakresleny obrazek, aby se mohl zpracovat a rozpoznat 
  */
  public int[][] doPole() {
    int p[][] = new int[sirka][vyska];
    handlepixels(offImage, 0, 0, sirkaPlatna, vyskaPlatna, p);
    return p;
  }
  
  /**
  * Zpracovava jeden pixel vstupniho obrazu. Dulezite jsou jen ty, ktere
  * lezi na souradnicich [xx5, yy5].
  */
  public int[][] handlesinglepixel(int x, int y, int pixel, int p[][]) {
    //int alpha = (pixel >> 24) & 0xff;
    //int red   = (pixel >> 16) & 0xff;
    //int green = (pixel >>  8) & 0xff;
    int blue  = (pixel      ) & 0xff;
    //Deal with the pixel as necessary... 
    if ( (((x+(velBodu/2))%velBodu)==0) && (((y+(velBodu/2))%velBodu)==0) )
      if (blue==0) 
        p[((x-(velBodu/2))/velBodu)][((y-(velBodu/2))/velBodu)]=1;  // "1" znamena cerny
      else 	
        p[((x-(velBodu/2))/velBodu)][((y-(velBodu/2))/velBodu)]=0;  // "0" znamena byli
    return p;    
  }
   
  /**
  * Metoda prevzata z helpu k JavaWorkShopu. Slouzi k prevodu obrazku 
  * z objketu Image do pole v pameti.
  */
  public int[][] handlepixels(Image img, int x, int y, int w, int h, int p[][]) {
    int[] pixels = new int[w * h];
    PixelGrabber pg = new PixelGrabber(img, x, y, w, h, pixels, 0, w);
    try {
      pg.grabPixels();
    }
    catch (InterruptedException e) {
      System.err.println("interrupted waiting for pixels!");
      return null;
    }
    if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
      System.err.println("image fetch aborted or errored");
      return null;
    }
    for (int j = 0; j < h; j++) 
      for (int i = 0; i < w; i++) 
        handlesinglepixel(x+i, y+j, pixels[j * w + i], p);
    return p;
  }
  
} // class Platno